# coding: utf-8

import json
from collections import defaultdict
from abejacli.config import (
    ORGANIZATION_ENDPOINT
)
from abejacli.session import (
    api_post
)
from abejacli.logger import get_logger
from abejacli.datalake import generate_channel_file_iter_by_period
from abejacli.config import DATASET_CHUNK_SIZE

logger = get_logger()


def create_request_element(channel_id, file_info, label_metadata_key):
    """
    create dataset item from datalake file

    :param channel_id:
    :param file_id:
    :param file_info:
    :param label_metadata_key:
    :return:
    """
    file_id = file_info.get('file_id')
    file_metadata = file_info.get('metadata')
    label = file_metadata['x-abeja-meta-{}'.format(label_metadata_key)]
    data_uri = 'datalake://{}/{}'.format(channel_id, file_id)

    data = {
        'source_data': [
            {
                'data_uri': data_uri,
                'data_type': file_info['content_type']
            }
        ],
        'attributes': {
            'classification': [
                {
                    label_metadata_key: label,
                }
            ]
        }
    }
    return data


def filter_items_by_max_size(dataset_items, max_size_for_label):
    """
    trim dataset items by max size for each lable value

    :param dataset_items:
    :param max_size:
    :return:
    """
    items_by_labels = defaultdict(list)
    # group by label value
    for item in dataset_items:
        label = item['attributes']['classification'][0]['label']
        items_by_labels[label].append(item)
    filtered_items = []
    for k, v in items_by_labels.items():
        items_size = len(v)
        # trim by max_size
        upload_size = items_size if max_size_for_label is None else min(max_size_for_label, items_size)
        if upload_size < items_size:
            logger.info('[Warning] Skipping {} items for label:{} to register'.format(items_size - upload_size, k))
        filtered_items += v[:upload_size]
    return filtered_items


def register_dataset_items(dataset_id, items):
    """
    execute dataset api to registr dataset items

    :param dataset_id:
    :param items:
    :return:
    """
    url = '{}/datasets/{}/items'.format(ORGANIZATION_ENDPOINT, dataset_id)

    def _chunked(l, n):
        for i in range(0, len(l), n):
            yield l[i:i + n]

    # max number of items for add items should is 500 (by default)
    for chunked_items in _chunked(items, DATASET_CHUNK_SIZE):
        api_post(url, json.dumps(chunked_items))


def register_dataset_items_from_datalake(dataset_id, channel_id, label_metadata_key, max_size_for_label):
    """
    register datasets from datalake channel

    :param dataset_id: target dataset id
    :param channel_id: target channel
    :param label_metadata_key: metadata key which label value is stored
    :param max_size_for_label: max size of dataset items for each label value
    :return:
    """
    print('Getting data from datalake....')
    file_iter = generate_channel_file_iter_by_period(channel_id)
    dataset_items = [
        create_request_element(channel_id, file_info, label_metadata_key)
        for file_info in file_iter
    ]
    print('Registering dataset items....')
    if max_size_for_label:
        dataset_items = filter_items_by_max_size(dataset_items, max_size_for_label)
    register_dataset_items(dataset_id, dataset_items)
    return {
        'result': 'success',
        'dataset_items': len(dataset_items),
        'dataset_id': dataset_id,
        'channel_id': channel_id
    }


def import_dataset_from_datalake(channel_id, dataset_id, label_metadata, _max):
    return register_dataset_items_from_datalake(dataset_id, channel_id, label_metadata, _max)
